require 'emcee/runner'
require 'emcee/node'
require 'emcee/agent'

class Emcee
  class Repository
    attr_reader :runner

    def initialize(options = {})
      @runner = Emcee::Runner.new(options)
      @agents = {}
      @nodes = {}
      build
    end

    def inspect
      YAML.dump(self)
    end

    def agent(agent_data)
      @agents["#{agent_data[:agent]} #{agent_data[:version]}"]
    end

    def node(node_name)
      @nodes[node_name]
    end

    def build
      runner.results.each do |result|
        node = Emcee::MCNode.new(result[:sender])
        @nodes[node.name] = node

        result[:data][:agents].each do |agent_data|
          agent_key = "#{agent_data[:agent]} #{agent_data[:version]}"
          existing_agent = @agents[agent_key]
          if existing_agent
            agent = existing_agent
          else
            agent = Emcee::Agent.new(runner, agent_data)
            @agents[agent_key] = agent
          end
          link(node, agent)
        end
      end
    end

    def link(node, agent)
      node.add_agent(agent)
      agent.add_node(node)
    end

    def agents
      @agents.values
    end

    def nodes
      @nodes.values
    end
  end
end
